package com.fs.erp.dto.sdk.df;


import com.alibaba.fastjson.JSON;
import com.fs.erp.dto.df.DFConfigVo;
import com.fs.erp.dto.sdk.df.enums.RequestUrlEnum;
import com.fs.his.config.FsSysConfig;
import com.fs.his.utils.ConfigUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * Wdt客户端
 */
@Component
@Slf4j
public class DfClient {
	@Autowired
	ConfigUtil configUtil;
	private final String baseUrl = "https://ds-api.sf-express.com/externalapi/";
	public static final String CHARSET = "UTF-8";
	public static final String CONTENT_TYPE = "application/json";


	private String getAppKey(int i) {
		FsSysConfig sysConfig = configUtil.getSysConfig();
		String dfConfigVo = sysConfig.getDfAccounts();
		List<DFConfigVo> dfConfigVos = JSON.parseArray(dfConfigVo, DFConfigVo.class);
		return dfConfigVos.get(i).getDfAppKey();
	}

	private String getAppsecret(int i) {
		FsSysConfig sysConfig = configUtil.getSysConfig();
		String dfConfigVo = sysConfig.getDfAccounts();
		List<DFConfigVo> dfConfigVos = JSON.parseArray(dfConfigVo, DFConfigVo.class);
		return dfConfigVos.get(i).getDfAppsecret();
	}




	private DfClient(){}

	/***
	 * description 生成签名
	 * @methodName getSign
	 * @param appSecret
	 * @param timestamp
	 * @param parameterString
	 * @param body
	 * @return java.lang.String
	 * @author 01398643
	 * @date 2020.12.29 下午 03:55
	 */
	private String getSign(String appSecret, String timestamp, String parameterString, String body) {
		if (StringUtils.isEmpty(parameterString)) {
			parameterString = "";
		}
		if (StringUtils.isEmpty(body)) {
			body = "";
		}
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(parameterString);
		stringBuilder.append(body);
		stringBuilder.append("&appSecret=").append(appSecret);
		stringBuilder.append("&timestamp=").append(timestamp);
		byte[] bytes = DigestUtils.sha512(stringBuilder.toString());
		return Hex.encodeHexString(bytes);
	}

	public String execute(RequestUrlEnum request, Map<String, Object> params,int i) throws IOException {
		String appkey = getAppKey(i);
		String appsecret = getAppsecret(i);
		String timestamp = String.valueOf(System.currentTimeMillis());
		String relativeUrl = request.getUrl();
		String type = request.getType();
		String url = baseUrl + relativeUrl;
		String sign = null;
		StringEntity stringEntity = null;
		log.info("开始执行请求，相对路径：{}，请求参数：{}", relativeUrl, params);
		if ("GET".equalsIgnoreCase(type)) {
			StringBuilder parameterString = new StringBuilder();
			for (String s : params.keySet()) {
				parameterString.append(s).append("=").append(params.get(s)).append("&");
			}
			if (parameterString.length() > 0) {
				parameterString.deleteCharAt(parameterString.length() - 1);
			}
			String parameter = parameterString.toString();
			url = url + "?" + parameter;
			sign = getSign(appsecret, timestamp, parameter, null);
		} else if ("POST".equals(type)) {
			String bodyJsonString = JSON.toJSONString(params);
			stringEntity = new StringEntity(bodyJsonString, ContentType.APPLICATION_JSON);
			sign = getSign(appsecret, timestamp, null, bodyJsonString);
		}
		log.info("计算签名完成, 签名为:{}",sign);
		//获取请求头
		Header[] headers = {new BasicHeader("Content-Type", CONTENT_TYPE),
				new BasicHeader("charset", CHARSET),
				new BasicHeader("appKey", appkey),
				new BasicHeader("sign", sign),
				new BasicHeader("timestamp", timestamp)};
		String jsonString = null;
		long startTime = System.currentTimeMillis();
		if ("GET".equalsIgnoreCase(type)) {
			HttpGet httpGet = new HttpGet(url.toString());
			httpGet.setHeaders(headers);
			HttpClient client = HttpClientBuilder.create().build();
			HttpResponse response = client.execute(httpGet);
			if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
				HttpEntity entity = response.getEntity();
				jsonString = EntityUtils.toString(entity, CHARSET);
				return jsonString;
			}
		} else if ("POST".equals(type)) {
			HttpPost httpPost = new HttpPost(url);
			httpPost.setHeaders(headers);
			httpPost.setEntity(stringEntity);
			HttpClient client = HttpClientBuilder.create().build();
			HttpResponse response = client.execute(httpPost);
			if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
				HttpEntity entity = response.getEntity();
				jsonString = EntityUtils.toString(entity, CHARSET);
				return jsonString;
			}
		}
		long endTime = System.currentTimeMillis();
		long costTime = endTime - startTime;
		// 打印 HTTP 请求的详细信息
		log.info("请求URL: {}, 请求体: {}, 请求头: {}, 响应体: {}, 耗时: {} ms", url, params, headers, jsonString, costTime);
		return null;
	}
}
